@@ -105,25 +105,38 @@ def checkDownload():
     if escape:
         raise StopProgram()
     if skip:
-        skip = False
         raise StopDownload()
 def checkEscape():
+    '''only called when it doesn't matter if a escape will happen, usually at the start of a iteration or the preparation phase'''
+    #so we can reset skip here, since it wont matter either way and will help remove false skip positives from the key event loop
+    global skip
+    skip = False
     global escape
     if escape:
         raise StopProgram()
-
 @asynccontextmanager
 async def lock_keys() -> None:
     '''blocks key echoing for this console and recognizes most keys
        including many combinations, user kill still works, alt+tab...
        it also serves as a quit program and skip download shortcut
+       from: https://python-prompt-toolkit.readthedocs.io/en/master/pages/asking_for_input.html
+       Since this is decorated with a asynccontextmanager, it's guarded by async with
+       https://docs.python.org/3/reference/compound_stmts.html#async-with
+       
+       since prompt_toolkit (3.0) input.attach attaches to a already running event asyncio event loop,
+       these key checks run essentially 'forever' in the same thread as the main one.
+       
+       This is event loop is not cooperative (although it's thread safe, since it's a single thread)
+       so stale 'skip' (not ctrl-c or escape, since those exit right away) can be set to 'True' with a
+       logical check error of set while a nondownload iteration was running then check on a unrelated one.
+       So then you should reset skip on the 'checkescape' function at the start of every iteration
     '''
     done = asyncio.Event()
     input = create_input()
     def keys_ready():
         global skip
         global escape
-        #escape needs flush in unix platforms, so this chain
+        #ctrl-c needs flush, so this chain
         for key_press in chain(input.read_keys(), input.flush_keys()):
             if key_press.key == 'escape' or key_press.key == 'c-c': #esc or control-c
                 escape = True
@@ -133,7 +146,10 @@ async def lock_keys() -> None:
 
     with input.raw_mode():
         with input.attach(keys_ready):
-            typer.echo(typer.style(f' Press escape to quit, and most other non-meta keys to skip downloads', bold=True))
+            #ignore keys in buffer before we are ready
+            input.read_keys()
+            input.flush_keys()
+            typer.echo(typer.style(f'Press escape to quit, and most other non-meta keys to skip downloads', bold=True))
             yield done
 
 #----------------non contextual str manipulation------------------------
@@ -155,11 +171,8 @@ def if_not_spaced_split_camelcase(name: str):
         name = ' '.join([s for s in re.split('([A-Z][^A-Z]*)', name) if s])
     return name
 
-#these two methods will be unnecessary once at python 3.9 is widespread in distros (ie: after ubuntu 20.04 is not supported)
-def removesuffix(name: str, suf: str):
-    if name.endswith(suf):
-        return name[:-len(suf)]
-    return name
+def removefirst(name: str, suf: str):
+    return name.replace(suf, '', 1)
 
 def removeprefix(name: str, pre: str):
     if name.startswith(pre):
@@ -222,36 +235,36 @@ def normalizer(t, nometa, hack):
     t = t.strip()
     #beginning and end definite articles in several european languages (people move them)
     #make sure we're only removing the capitalized start and end forms with spaces
-    t = removesuffix(t, ', The')
+    t = removefirst(t, ', The')
     t = removeprefix(t, 'The ')
-    t = removesuffix(t, ', Los')
+    t = removefirst(t, ', Los')
     t = removeprefix(t, 'Los ')
-    t = removesuffix(t, ', Las')
+    t = removefirst(t, ', Las')
     t = removeprefix(t, 'Las ')
-    t = removesuffix(t, ', Les')
+    t = removefirst(t, ', Les')
     t = removeprefix(t, 'Les ')
-    t = removesuffix(t, ', Le')
+    t = removefirst(t, ', Le')
     t = removeprefix(t, 'Le ')
-    t = removesuffix(t, ', La')
+    t = removefirst(t, ', La')
     t = removeprefix(t, 'La ')
-    t = removesuffix(t, ', L\'')
+    t = removefirst(t, ', L\'')
     #L' sometimes ommits the space so always remove L' at the start even without space
     t = removeprefix(t, 'L\'')  #if there is a extra space the next join will remove it
-    t = removesuffix(t, ', Der')
+    t = removefirst(t, ', Der')
     t = removeprefix(t, 'Der ')
-    t = removesuffix(t, ', Die')
+    t = removefirst(t, ', Die')
     t = removeprefix(t, 'Die ')
-    t = removesuffix(t, ', Das')
+    t = removefirst(t, ', Das')
     t = removeprefix(t, 'Das ')
-    t = removesuffix(t, ', El')
+    t = removefirst(t, ', El')
     t = removeprefix(t, 'El ')
-    t = removesuffix(t, ', Os')
+    t = removefirst(t, ', Os')
     t = removeprefix(t, 'Os ')
-    t = removesuffix(t, ', As')
+    t = removefirst(t, ', As')
     t = removeprefix(t, 'As ')
-    t = removesuffix(t, ', O')
+    t = removefirst(t, ', O')
     t = removeprefix(t, 'O ')
-    t = removesuffix(t, ', A')
+    t = removefirst(t, ', A')
     t = removeprefix(t, 'A ')
     #Tries to make roman numerals in the range 1-20 equivalent to normal numbers (to handle names that change it).
     #If both sides are roman numerals there is no harm done if XXIV gets turned into 204 in both sides.
@@ -389,6 +402,9 @@ def getDirectoryPath(cfg: Path, setting: str):
     dirp = os.path.expanduser(configParser['DUMMY'][setting].strip('\t ').strip('"'))
     return Path(dirp)
 
+def error(error: str):
+    typer.echo(typer.style(error, fg=typer.colors.RED, bold=True))
+
 def test_common_errors(cfg: Path, playlist: str, system: str):
     '''returns a tuple with (playlist_dir: Path, thumbnail_dir: Path, PLAYLISTS: [Path], SYSTEMS: [str]) '''
     global viewer
@@ -396,22 +412,22 @@ def test_common_errors(cfg: Path, playlist: str, system: str):
     if not viewer:
         typer.echo(f'Shell image viewer chafa was not found')
     if not cfg or not cfg.is_file():
-        typer.echo(f'Invalid Retroarch cfg file: {cfg}')
+        error(f'Invalid Retroarch cfg file: {cfg}')
         raise typer.Exit(code=1)
     thumbnails_directory = getDirectoryPath(cfg, 'thumbnails_directory')
-    if not thumbnails_directory.is_dir():
-        typer.echo(f'Invalid Retroarch thumbnails directory: {thumbnails_directory}')
+    if not thumbnails_directory.is_dir() or not os.access(thumbnails_directory, os.W_OK):
+        error(f'Invalid Retroarch thumbnails directory: {thumbnails_directory}')
         raise typer.Exit(code=1)
     playlist_dir = getDirectoryPath(cfg, 'playlist_directory')
-    if not playlist_dir.is_dir():
-        typer.echo(f'Invalid Retroarch playlist directory: {playlist_dir}')
+    if not playlist_dir.is_dir() or not os.access(playlist_dir, os.R_OK):
+        error(f'Invalid Retroarch playlist directory: {playlist_dir}')
         raise typer.Exit(code=1)
-    PLAYLISTS = list(playlist_dir.glob('./*.lpl'))
+    PLAYLISTS = [ pl for pl in playlist_dir.glob('./*.lpl') if pl.is_file() and os.access(pl, os.R_OK) ]
     if not PLAYLISTS:
-        typer.echo(f'Retroarch cfg file has empty playlist directory: {playlist_dir}')
+        error(f'Invalid playlist files in playlist directory: {playlist_dir}')
         raise typer.Exit(code=1)
     if playlist and Path(playlist_dir, playlist) not in PLAYLISTS:
-        typer.echo(f'Unknown user provided playlist: {playlist}')
+        error(f'Invalid user provided playlist: {playlist}')
         raise typer.Exit(code=1)
 
     try:
@@ -420,10 +436,10 @@ def test_common_errors(cfg: Path, playlist: str, system: str):
             soup = BeautifulSoup(page.text, 'html.parser')
         SYSTEMS = [ unquote(node.get('href')[:-1]) for node in soup.find_all('a') if node.get('href').endswith('/') and not node.get('href').endswith('../') ]
     except (RequestError,HTTPStatusError) as err:
-        typer.echo(f'Could not get the remote thumbnail system names')
+        error(f'Could not get the remote thumbnail system names, exiting')
         raise typer.Exit(code=1)
     if system and system not in SYSTEMS:
-        typer.echo(f'The user provided system name {system} does not match any remote thumbnail system names')
+        error(f'The user provided system name {system} does not match any remote thumbnail system names')
         raise typer.Exit(code=1)
     return (playlist_dir, thumbnails_directory, sorted(PLAYLISTS), sorted(SYSTEMS))
 
@@ -480,10 +496,10 @@ def mainfuzzsingle(cfg: Path = typer.Argument(CONFIG, help='Path to the retroarc
                     names = readPlaylistAndPrepareDirectories(Path(playlist_dir, playlist), tmpdir, thumbnails_dir)
                     await downloader(names,system,wait_before,wait_after,filters,noimage,nomerge,nofail,nometa,hack,nosubtitle,verbose,before,tmpdir,thumbnails_dir,client)
         except StopPlaylist as e:
-            typer.echo(typer.style(f'Cloudflare is down for {system}', fg=typer.colors.RED, bold=True))
+            error(f'Cloudflare is down for {system}')
             raise typer.Exit(code=1)
         except StopProgram as e:
-            typer.echo(f'Cancelled by user')
+            error(f'Cancelled by user, exiting')
             raise typer.Exit()
     asyncio.run(runit(), debug=False)
 
@@ -519,9 +535,9 @@ def mainfuzzall(cfg: Path = typer.Argument(CONFIG, help='Path to the retroarch c
                         try:
                             await downloader(names,system,wait_before,wait_after,filters,noimage,nomerge,nofail,nometa,hack,nosubtitle,verbose,before,tmpdir,thumbnails_dir,client)
                         except StopPlaylist as e:
-                            typer.echo(typer.style(f'Cloudflare is down for {system}', fg=typer.colors.RED, bold=True))
+                            error(f'Cloudflare is down for {system}')
         except StopProgram as e:
-            typer.echo(f'Cancelled by user')
+            error(f'Cancelled by user, exiting')
             raise typer.Exit()
     asyncio.run(runit(), debug=False)
 
@@ -552,7 +568,7 @@ async def downloadgamenames(client, system):
                 l1 = { unquote(Path(node.get('href')).name[:-4]) : lr_thumb+node.get('href') for node in soup.find_all('a') if node.get('href').endswith('.png')}
             args.append(l1)
     except (RequestError,HTTPStatusError) as err:
-        typer.echo(str(err))
+        error(f'Could not get the remote thumbnail game names, exiting')
         raise typer.Exit(code=1)
     return args
 
@@ -738,12 +754,9 @@ async def download(client, url, destination, download_format, missing_format, wa
                             checkDownload()
                             w.write(chunk)
             return True
-        except IOError as e:
-            typer.echo(str(e))
-            raise typer.Exit(code=1)            
         except (RequestError,HTTPStatusError) as e:
             if max_retries <= 0:
-                typer.echo(str(e))
+                error(f'Download max retries exceeded, exiting')
                 raise typer.Exit(code=1)
             max_retries -= 1
 
@@ -823,5 +836,5 @@ def fuzzall():
     typer.run(mainfuzzall)
 
 if __name__ == "__main__":
-    typer.echo('Please run libretro-fuzz or libretro-fuzzall instead of running the script directly')
+    error('Please run libretro-fuzz or libretro-fuzzall instead of running the script directly')
     raise typer.Exit(code=1)
